@@ -19,6 +19,7 @@ type Backend struct {
 	Settings     gonetworkmanager.Settings
 	Connections  map[string]gonetworkmanager.Connection
 	AccessPoints map[string]gonetworkmanager.AccessPoint
+	Device       gonetworkmanager.DeviceWireless
 }
 
 // New creates a new dbus.Backend.
@@ -42,7 +43,10 @@ func New() (wifi.Backend, error) {
 }
 
 func (b *Backend) getWirelessDevice() (gonetworkmanager.DeviceWireless, error) {
-	// TODO: Cache result in struct?
+	if b.Device != nil {
+		return b.Device, nil
+	}
+
 	devices, err := b.NM.GetDevices()
 	if err != nil {
 		return nil, err
@@ -50,6 +54,7 @@ func (b *Backend) getWirelessDevice() (gonetworkmanager.DeviceWireless, error) {
 
 	for _, device := range devices {
 		if dev, ok := device.(gonetworkmanager.DeviceWireless); ok {
+			b.Device = dev
 			return dev, nil
 		}
 	}
@@ -91,7 +96,12 @@ func (b *Backend) BuildNetworkList(shouldScan bool) ([]wifi.Connection, error) {
 		return nil, err
 	}
 
-	var conns []wifi.Connection
+	// We use a map with a composite key {ssid, security} to store unique connections.
+	type connKey struct {
+		ssid     string
+		security wifi.SecurityType
+	}
+	uniqueConns := make(map[connKey]wifi.Connection)
 	processedSSIDs := make(map[string]bool)
 
 	activeConnections, err := b.NM.GetPropertyActiveConnections()
@@ -122,6 +132,16 @@ func (b *Backend) BuildNetworkList(shouldScan bool) ([]wifi.Connection, error) {
 		}
 
 		strength, _ := ap.GetPropertyStrength()
+		hwAddress, _ := ap.GetPropertyHWAddress()
+		frequency, _ := ap.GetPropertyFrequency()
+
+		wifiAP := wifi.AccessPoint{
+			SSID:      ssid,
+			BSSID:     hwAddress,
+			Strength:  strength,
+			Frequency: uint(frequency),
+		}
+
 		if existing, exists := newAccessPoints[ssid]; exists {
 			exStrength, _ := existing.GetPropertyStrength()
 			if strength <= exStrength {
@@ -145,6 +165,79 @@ func (b *Backend) BuildNetworkList(shouldScan bool) ([]wifi.Connection, error) {
 			security = wifi.SecurityOpen
 		}
 
+		key := connKey{ssid: ssid, security: security}
+
+		// Check if we already have this Connection processed
+		if conn, exists := uniqueConns[key]; exists {
+			// We already have a base connection for this SSID/Security pair.
+			// Just add the AP. AddAccessPoint handles merging logic if needed,
+			// but since we keyed by what Compare checks, we can trust it matches.
+			_ = conn.AddAccessPoint(wifi.Connection{
+				SSID:         ssid,
+				Security:     security,
+				AccessPoints: []wifi.AccessPoint{wifiAP},
+				// Other fields like Active/Known are merged by AddAccessPoint
+				// We need to populate them if this specific AP instance has them "better" or "true"
+				// But IsKnown is per-SSID (mostly), IsActive is per-connection.
+				// Let's populate the temp struct with what we know from this AP context.
+			})
+
+			// Let's reconstruct the temp connection properly to pass to AddAccessPoint
+			tempConn := wifi.Connection{
+				SSID:         ssid,
+				Security:     security,
+				IsSecure:     isSecure,
+				IsVisible:    true,
+				AccessPoints: []wifi.AccessPoint{wifiAP},
+			}
+
+			var knownConn gonetworkmanager.Connection
+			for _, kc := range knownConnections {
+				s, err := kc.GetSettings()
+				if err != nil {
+					continue
+				}
+				if wireless, ok := s["802-11-wireless"]; ok {
+					if ssidBytes, ok := wireless["ssid"].([]byte); ok {
+						if string(ssidBytes) == ssid {
+							knownConn = kc
+							break
+						}
+					}
+				}
+			}
+
+			if knownConn != nil {
+				s, _ := knownConn.GetSettings()
+				var id string
+				var lastConnected *time.Time
+				if c, ok := s["connection"]; ok {
+					if i, ok := c["id"].(string); ok {
+						id = i
+					}
+					if ts, ok := c["timestamp"].(uint64); ok && ts > 0 {
+						t := time.Unix(int64(ts), 0)
+						lastConnected = &t
+					}
+				}
+				autoConnect := true
+				if c, ok := s["connection"]; ok {
+					if ac, ok := c["autoconnect"].(bool); ok {
+						autoConnect = ac
+					}
+				}
+				tempConn.IsKnown = true
+				tempConn.IsActive = activeConnectionID != "" && id == activeConnectionID
+				tempConn.LastConnected = lastConnected
+				tempConn.AutoConnect = autoConnect
+			}
+
+			// Now merge
+			_ = conn.AddAccessPoint(tempConn)
+			uniqueConns[key] = conn
+			continue
+		}
+
 		var connInfo wifi.Connection
 		var knownConn gonetworkmanager.Connection
 		for _, kc := range knownConnections {
@@ -188,10 +281,10 @@ func (b *Backend) BuildNetworkList(shouldScan bool) ([]wifi.Connection, error) {
 				IsKnown:       true,
 				IsSecure:      isSecure,
 				IsVisible:     true,
-				Strength:      strength,
 				Security:      security,
 				LastConnected: lastConnected,
 				AutoConnect:   autoConnect,
+				AccessPoints:  []wifi.AccessPoint{wifiAP},
 			}
 		} else {
 			connInfo = wifi.Connection{
@@ -199,12 +292,18 @@ func (b *Backend) BuildNetworkList(shouldScan bool) ([]wifi.Connection, error) {
 				IsKnown:     false,
 				IsSecure:    isSecure,
 				IsVisible:   true,
-				Strength:    strength,
 				Security:    security,
 				AutoConnect: false, // Can't autoconnect to a network we don't know
+				AccessPoints: []wifi.AccessPoint{wifiAP},
 			}
 		}
-		conns = append(conns, connInfo)
+		uniqueConns[key] = connInfo
+	}
+
+	// Now build the final list from uniqueConns
+	var conns []wifi.Connection
+	for _, c := range uniqueConns {
+		conns = append(conns, c)
 	}
 
 	for _, knownConn := range knownConnections {
